import pytest

from pysj.formats import all_image_extensions, common_image_extensions


def test_common_formats():

    assert common_image_extensions == {
        "png",
        "jpg",
        "jpeg",
        "jfif",
        "j2p",
        "jpx",
        "j2k",
        "gif",
        "bmp",
        "dds",
        "dib",
        "eps",
        "icns",
        "im",
        "msp",
        "pcx",
        "apng",
        "pbm",
        "pgm",
        "ppm",
        "pnm",
        "sgi",
        "spi",
        "tga",
        "tiff",
        "tif",
        "webp",
        "xbm",
    }


def test_all_image_formats():
    len(all_image_extensions) >= 119



from pysj import flatten


def test_basic_nested_list():

    assert flatten([[1, 2, 3], [4, 5, 6]]) == [1, 2, 3, 4, 5, 6]


def test_nested_list_with_different_types():

    assert flatten(
        [[(1.3, 3.0), "string", "another string"], [4, 5e2, sorted({"a", "b", "c"})]]
    ) == [1.3, 3.0, "string", "another string", 4, 5e2, "a", "b", "c"]


def test_already_flat_input():

    flat_input = [8, 7, 6, 5, 4, 3, 2, 1]
    assert flatten(flat_input) == flat_input



import pytest
from pytest import approx

from pysj import take


def test_take_4():
    assert list(take(4, "abcdefgh")) == list("abcd")


def test_take_2():
    assert list(take(2, range(10))) == [0, 1]



import dataclasses
import json
import time
from datetime import date, datetime, timedelta
from fractions import Fraction
from itertools import islice, tee, zip_longest
from typing import Any, Callable, Iterable, List, Tuple

NUMPY_SUPPORT_FLAG = True
try:
    import numpy as np
except ImportError:
    NUMPY_SUPPORT_FLAG = False


class Timer:
    """Simple class working like a stopwatch."""

    def __init__(self) -> None:

        self.time = None
        self.start_time = time.perf_counter()

    def __enter__(self):
        print("Starting timer")
        self.start()

    def __exit__(self, *_):
        print("Elapsed time", self.lap(), "s.")

    def start(self):
        self.time = time.perf_counter()

    def reset(self):
        end_time = self.lap()
        self.start()
        return end_time

    def lap(self):
        return time.perf_counter() - self.time

    def split(self):
        """Alias for lap"""
        return self.lap()

    def total(self):
        return time.perf_counter() - self.start_time


def isotime(precision="d", dt=None):
    """Get the current date/time in isoformat. Default precision is day, accepts d(ay), (h)our, (m)inute, (s)econd.
    Per defualt the current time is used, this can be overriden by supplying a datetime object with the *dt* kwarg"""
    if not dt:
        dt = datetime.now()
    precision_map = {
        "d": slice(0, 10),
        "h": slice(0, 13),
        "m": slice(0, 16),
        "s": slice(0, 19),
    }

    return dt.isoformat()[precision_map.get(precision[0])]


def seconds(
    days=0,
    seconds=0,
    microseconds=0,
    milliseconds=0,
    minutes=0,
    hours=0,
    weeks=0,
    years=0,
    months=0,
    quarters=0,
) -> int:
    """Returns the number of seconds in the given time intervals combined, rounded down to nearest integer.

    This is a short variant of `int(timedelta(**kwargs).total_seconds())` with a
    few extra named time intervals(year, month, quarter)

    A year is calculated as 365 days, a month is 30 days, a quarter is 1/4 of a year
    """

    days += float((Fraction(365, 4) * quarters))
    days += months * 30
    days += years * 365

    return int(
        timedelta(
            days, seconds, microseconds, milliseconds, minutes, hours, weeks
        ).total_seconds()
    )


def flatten(iterable: Iterable) -> list:
    """Recursively flattens an iterable (depth first)

    Usage
    -----

    >>> nested_list = [[(1.3, 3.0), "string", "another string"], [4, 5E2, sorted({"a", "b", "c"})]]
    >>> print(flatten(nested_list))
    [1.3, 3.0, "string", "another string", 4, 5E2, "a", "b", "c"]

    """

    def _recursively_flatten(iterable: Iterable):
        for e in iterable:
            if isinstance(e, Iterable) and not isinstance(e, str):
                yield from _recursively_flatten(e)
            else:
                yield e

    return list(_recursively_flatten(iterable))


def transpose(iterable: Iterable[Iterable]) -> List[List]:
    """Transpose a list of lists with dimensions X=Y."""

    return [list(x) for x in zip(*iterable)]


class ExtendedJSONEncoder(json.JSONEncoder):
    """Subclass for `json.JSONEncoder` making encoding of datetime, dataclasses and numpy arrays work.

    Usage
    -----
    >>> obj = {"timestamp": datetime.today()}
    >>> json.dumps(obj, cls=ExtendedJSONEncoder)

    """

    def default(self, o):
        if isinstance(o, datetime):
            return o.isoformat(timespec="seconds")
        if isinstance(o, date):
            return o.isoformat()
        if dataclasses.is_dataclass(o):
            return dataclasses.asdict(o)

        if NUMPY_SUPPORT_FLAG and type(o) in [np.int8, np.int16, np.int32, np.int64]:
            return int(o)

        return super().default(o)


def _extended_JSON_decoder_object_hook(d):
    for k, v in d.items():
        if isinstance(v, dict):
            _extended_JSON_decoder_object_hook(v)
        else:
            if isinstance(v, str):
                if any(
                    [
                        v.startswith("19"),
                        v.startswith("20"),
                    ]
                ):
                    try:
                        d[k] = datetime.fromisoformat(v)
                    except ValueError:
                        pass  # Invalid isoformat string, leave as is
    return d


class ExtendedJSONDecoder(json.JSONDecoder):
    def __init__(self):
        super().__init__(object_hook=_extended_JSON_decoder_object_hook)


class ConfigurableJSONTranscoder:
    def decoder(self):
        pass

    def encoder(self):
        pass


def paginate(
    url: str,
    pagesize=500,
    data_attribute_name=None,
    max_num_requests=10_000,
    start_one=False,
    offset_is_n_of_elements=False,
):
    """Function returning a function yielding new urls if the data seems valid

    Usage
    -----

        get_url = paginate("lol.lol?offset=OFFSET&pagesize=PAGESIZE", max_num_requests=20, start_one=False)

        data = None
        while url := get_url(data):
            data = requests.get(url)
            # Do someting with the data, get_url will yield a new url a long as data is set to
            # someting looking like a valid json response without an error attribute,
            # and as long the data don't repeat.

    """
    n = -1
    offset_multiplier = 1
    if offset_is_n_of_elements:
        offset_multiplier = pagesize

    if start_one:
        n = 0
    else:
        max_num_requests -= 1  # Zero indexed

    last_response = "not_json"

    def inner_page(data=None):
        nonlocal n
        if (
            data := _extract_json_data(data, data_attribute_name)
            and n < max_num_requests
            and last_response != data
        ):
            n += 1
            return url.replace("OFFSET", str(n * offset_multiplier)).replace(
                "PAGESIZE", str(pagesize)
            )

    def _extract_json_data(data, data_attr=None):
        """Extracts and returns items in json response, or False if there are no more items, returns True if data is None"""

        if isinstance(data, dict):
            if data.get("error"):
                return False
            if data_attr:
                data = data.get(data_attr)
            else:
                if data.get("data"):
                    data = data.get("data")
                elif data.get("items"):
                    data = data.get("items")

        if data is None:
            return True
        elif data:
            return data
        else:
            return False

    return inner_page


def n_wise(n: int, data: Iterable):
    """Yields *n* items from data like a gliding window, as long as there is enough elements in data to yield the full window.
    If n == 3, and data = (1,2,3,4,5) the following items are yielded: (1,2,3), (2,3,4), (3,4,5).

    Like itertools.pairwise, but generalized to *n* items.
    """
    iterators = tee(
        data, n
    )  # Using tee on generators is not a good idéa if they are consumed very out of sync,  as all the intermidiate items must be cached.

    for i, iterator in enumerate(iterators):
        for _ in range(
            i
        ):  # But in this case the tee'd generators are never more out of sync than the length of the window (*n*)
            next(iterator, None)

    return zip(*iterators)


def triplewise(data: Iterable):
    return n_wise(3, data)


def moving_average(window_size: int, data: Iterable):
    yield from (sum(x) / window_size for x in n_wise(window_size, data))


def take(n: int, iterable: Iterable):
    """Return first n items of the *iterable*"""
    return islice(iterable, n)


def first(iterable: Iterable):
    """Return first item of the *iterable*"""
    iterable_ = iter(iterable)
    return next(iterable_)


def chunk(n, iterable: Iterable, fillvalue=None):
    """Yields chunks of size *n* from *iterable*. Last chunk are filled with *fillvalue* if size of *iterable* is not a multiple of *n*."""
    args = [iter(iterable)] * n
    return zip_longest(*args, fillvalue=fillvalue)



from pysj import paginate


def test_paginated():
    i = 0
    get_url = paginate(
        "lol.lol?offset=OFFSET&pagesize=PAGESIZE", max_num_requests=20, start_one=False
    )

    data = None
    while url := get_url(data):
        i += 1
        if i < 10:
            data = {"valid": i}
        print(url)



def task_format():
    return {
        "actions": ["black .", "isort ."],
        "verbosity": 2,
    }


def task_test():
    return {
        "actions": ["tox -e py310"],
        "verbosity": 2,
    }


def task_fulltest():
    return {
        "actions": ["tox --skip-missing-interpreters"],
        "verbosity": 2,
    }


def task_build():
    return {
        "actions": ["flit build"],
        "task_dep": ["precommit"],
        "verbosity": 2,
    }


def task_publish():
    return {
        "actions": ["flit publish"],
        "task_dep": ["build"],
        "verbosity": 2,
    }


def task_precommit():
    return {"actions": None, "task_dep": ["format", "fulltest"]}



""" Pysj makes Python development more comfortable, with utils, classes and helper
functions I find myself reimplementing in several projects."""
__version__ = "1.0"

from .crypto import md5, sha1, sha256, uuid
from .main import (ExtendedJSONDecoder, ExtendedJSONEncoder, Timer, chunk,
                   first, flatten, isotime, moving_average, n_wise, paginate,
                   seconds, take, transpose, triplewise)



import string

import pytest

from pysj import md5, sha1, sha256, uuid
from pysj.crypto import ALPHABET


@pytest.mark.parametrize(
    "hash_algorithm, digest",
    [
        ("sha256", "9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08"),
        ("sha1", "a94a8fe5ccb19ba61c4c0873d391e987982fbbd3"),
        ("md5", "098f6bcd4621d373cade4e832627b4f6"),
    ],
)
def test_hashing(hash_algorithm, digest):
    assert eval(f"{hash_algorithm}('test')") == digest


def test_uuid_generation():
    assert len(uuid()) == 36 and all([x in string.hexdigits + "-" for x in uuid()])


def test_uuid_alpha_generation():
    assert 20 < len(uuid("alpha")) < 24 and all([x in ALPHABET for x in uuid("alpha")])


def test_uuid_int_generation():
    assert isinstance(uuid("int"), int)


def test_force_mutable():
    assert sha256([1, 2, 3], force_mutable=True) == sha256("[1, 2, 3]")


def test_force_mutable_error_when_flag_is_false():
    with pytest.raises(AttributeError):
        assert sha256([1, 2, 3])



import pytest
from pytest import approx

from pysj import n_wise, triplewise

test_data = (1, 2, 3, 4, 5, 6)


def test_n_wise_2_comp_pairwise():

    try:
        from itertools import pairwise

        #  itertools.pairwise and n_wise with a n of 2 should be equal.
        assert sum(sum((a, b)) for a, b in pairwise(test_data)) == sum(
            sum((a, b)) for a, b in n_wise(2, test_data)
        )
    except ImportError:
        pass  # pairwise was added to itertools in Python 3.10


def test_n_wise_triplewise():
    assert sum(sum(t) for t in triplewise(test_data)) == 42


def test_n_wise_3():
    assert sum(sum(t) for t in n_wise(3, test_data)) == 42


def test_n_wise_4():
    assert sum(sum(t) for t in n_wise(4, test_data)) == 42


def test_n_wise_5():
    assert sum(sum(t) for t in n_wise(5, test_data)) == 35



from pysj import seconds


def test_hours():
    assert seconds(hours=1) == 3600


def test_quarters():
    assert seconds(quarters=2) == 15768000  # 182 days and 12 hours


def test_milliseconds_less_than_a_second():
    assert seconds(milliseconds=400) == 0


def test_milliseconds_more_than_a_second_less_than_two():
    assert seconds(milliseconds=1999) == 1


def test_large_return_value():
    assert seconds(years=1999) == 63040464000


def test_time_interval_composition():
    assert (
        seconds(
            hours=1,
            months=1,
            days=1,
            years=1,
            minutes=1,
            weeks=1,
            quarters=1,
            seconds=1,
        )
        == 42706861
    )


def test_interval_comparison_quarter_in_year():
    assert seconds(quarters=4) == seconds(years=1)


def test_interval_comparison_days_in_year():
    assert seconds(days=365) == seconds(years=1)



import hashlib
import json
import uuid as _uuid
from typing import Literal, Union

ALPHABET = "ABCDEFGHJKLMNPQRSTUVWXYZ" "abcdefghijkmnpqrstuvwxyz" "23456789"


def _hash(plaintext: str, algorithm="sha256", force_mutable=False) -> str:
    """Calculate hashdigest of plaintext using any algorithm in `hashlib.algorithms_guaranteed`"""

    if force_mutable:
        plaintext = __force_mutable(plaintext)

    if algorithm not in hashlib.algorithms_guaranteed:
        raise NotImplementedError(f"Hash algorithm {algorithm} is not implemented.")

    hash_fn = getattr(hashlib, algorithm)
    return hash_fn(plaintext.encode()).hexdigest()


def sha256(plaintext: str, force_mutable=False) -> str:
    """Returns the hexdigest representation of the sha256 hash from the plaintext input"""
    return _hash(plaintext, "sha256", force_mutable)


def sha1(plaintext: str, force_mutable=False) -> str:
    """Returns the hexdigest representation of the sha1 hash from the plaintext input"""
    return _hash(plaintext, "sha1", force_mutable)


def md5(plaintext: str, force_mutable=False) -> str:
    """Returns the hexdigest representation of the md5 hash from the plaintext input"""
    return _hash(plaintext, "md5", force_mutable)


def __force_mutable(plaintext):
    """Dumps mutable objects to string to make them hashable"""
    if not isinstance(plaintext, str):
        return json.dumps(plaintext)
    return plaintext


def uuid(type: Literal["alpha", "int"] = None, length: int = None) -> Union[str, int]:
    if length:
        raise NotImplementedError("Not implemented yet")
    if type == "alpha":
        alphabet: str = ALPHABET
        number: int = _uuid.uuid4().int
        output = ""
        length = len(alphabet)
        while number:
            number, digit = divmod(number, length)
            output += alphabet[digit]
        return output[::-1]
    elif type == "int":
        return _uuid.uuid4().int
    else:
        return str(_uuid.uuid4())



import json
import logging
import socketserver
from datetime import datetime, timedelta
from multiprocessing import Process, Queue
from pathlib import Path
from queue import Empty
from time import sleep, time
from urllib.parse import parse_qs, urlsplit

from numpy import true_divide

logging.basicConfig(level=logging.DEBUG)
log = logging.getLogger(__name__)


class WebhookTCPHandler(socketserver.BaseRequestHandler):
    def handle(self):
        data = self.request.recv(2048).strip()

        url = data.decode("utf-8").split(" HTTP/")[0][4:].strip()  # Get url of request
        resource, queryparams = urlsplit(url).path[1:], parse_qs(urlsplit(url).query)

        self.server.accepted_webhooks.put((resource, queryparams))
        log.debug("Webhook received")
        self.request.sendall(
            b"HTTP/1.1 201 ACCEPTED\r\nCache-Controll: no-cache\r\nConnection: close\r\n\r\n"
        )


class WebhookTCPServer(socketserver.TCPServer):
    def __init__(self, server_address, request_handler, accepted_webhooks):
        socketserver.TCPServer.__init__(self, server_address, request_handler)
        self.accepted_webhooks = accepted_webhooks
        self.allow_reuse_address = True
        self.server_activate()


def webhook_server(accepted_webhooks):
    """Minimal webserver accepting webhooks"""
    with WebhookTCPServer(
        ("0.0.0.0", 4488), WebhookTCPHandler, accepted_webhooks
    ) as server:
        server.allow_reuse_address = True
        server.server_activate()
        server.serve_forever()


class SchedulableProcess:
    def __init__(self, target=None, args=None, name=None, **kwargs):

        self.target = target
        self.args = args if args else ()
        self.triggers = kwargs
        self.name = name if name else target.__name__
        self.proc = None
        self.last_run = None

    def start(self, args=None):
        args = args if args else self.args
        log.info(f"Running process {self.name}")
        self.proc = Process(target=self.target, args=args, name=self.name)
        self.last_run = datetime.now()
        self.proc.start()

    def tick(self):
        if not self.proc or not self.proc.is_alive():
            if not self.triggers:
                self.start()
                return
            for trigger, value in self.triggers.items():
                if trigger == "after":
                    if self.last_run is None:
                        if datetime.fromisoformat(value) < datetime.now():
                            self.start()
                    elif trigger == "interval":
                        if (
                            self.last_run is None
                            or self.last_run + timedelta(seconds=value) < datetime.now()
                        ):
                            self.start()
                    elif trigger == "webhook":
                        pass  # Webhook triggers are handled in the event loop
                    else:
                        log.info(f"Unknown trigger ignored. ({trigger}).")

    def write_last_run_to_file(self):
        """TODO: Create save config func (e.g. last_run)"""
        config = json.load(Path("~/.overengineer").expanduser())
        if config_update := config.get(self.name):
            self.args = self.args + config_update


class OverEngineer:
    """
    TODO:
    - systemd service script that makes file paths to file/console
    - script entrypoint to script creating systemd unitfile
    - accept a file or folder to watch for changes, then run a task
    """

    def __init__(self):
        self.accepted_webhooks = Queue()

        self.procs = [
            SchedulableProcess(target=webhook_server, args=(self.accepted_webhooks,))
        ]

    def register(self, *schedulable_processes):
        self.procs.extend(schedulable_processes)

    def start_event_loop(self):
        try:
            while True:  # Event loop
                for process in self.procs:
                    process.tick()

                if not self.accepted_webhooks.empty():  # There are cookies
                    try:
                        resource, args = self.accepted_webhooks.get_nowait()
                        for process in self.procs:
                            if process.name == resource:
                                log.info(
                                    f"Processing webhook (1/{self.accepted_webhooks.qsize()+1}, {resource}, {args}"
                                )
                                process.start(args=args.get("arg"))
                                break

                    except Empty:
                        pass  # They lied
                sleep(1)
        except KeyboardInterrupt:
            self.shutdown()

    def shutdown(self):
        for process in [x.proc for x in self.procs]:
            if (
                process and process.is_alive()
            ):  # Running is_alive has a side effect of joining proc if finished
                process.terminate()
                process.join(1)
                if process and process.is_alive():
                    process.kill()
                    process.join(1)



import pytest

from pysj import moving_average

test_data = (1, 2, 3, 4, 5, 6)


def test_moving_average_2():
    assert tuple(moving_average(2, test_data)) == (1.5, 2.5, 3.5, 4.5, 5.5)


def test_moving_average_3():
    assert tuple(moving_average(3, test_data)) == (2, 3, 4, 5)


def test_moving_average_5():
    assert tuple(moving_average(5, test_data)) == (3, 4)



import datetime
import json
from dataclasses import dataclass

import numpy as np

from pysj import ExtendedJSONDecoder, ExtendedJSONEncoder


def test_date_json_encoding():

    test = json.dumps(
        {"lol": datetime.date(2021, 12, 1)},
        cls=ExtendedJSONEncoder,
    )

    assert test == '{"lol": "2021-12-01"}'


def test_datetime_json_encoding():

    test = json.dumps(
        {"lol": datetime.datetime.fromisoformat("2021-12-01T04:50:00.123456")},
        cls=ExtendedJSONEncoder,
    )

    assert test == '{"lol": "2021-12-01T04:50:00"}'


def test_datetime_json_decoding():

    test = json.loads(
        '{"lol": "2021-12-01T04:50:00"}',
        cls=ExtendedJSONDecoder,
    )

    assert test == {"lol": datetime.datetime.fromisoformat("2021-12-01T04:50:00")}


def test_numpy_serialization():

    test = json.dumps(
        [1, 2, 3, np.int64(10), np.int8(11), np.int32(12)],
        cls=ExtendedJSONEncoder,
    )

    assert test == "[1, 2, 3, 10, 11, 12]"


def test_dataclass_serialization():
    @dataclass
    class TestDataClass:
        field_1: str
        field_2: int
        field_3: str

    test = json.dumps(
        TestDataClass("lol", 1234, "kek"),
        cls=ExtendedJSONEncoder,
    )

    assert test == '{"field_1": "lol", "field_2": 1234, "field_3": "kek"}'



import pytest
from pytest import approx

from pysj import chunk


def test_chunking_of_string():
    assert list(chunk(4, "my test string is a multiple of 4...")) == [
        ("m", "y", " ", "t"),
        ("e", "s", "t", " "),
        ("s", "t", "r", "i"),
        ("n", "g", " ", "i"),
        ("s", " ", "a", " "),
        ("m", "u", "l", "t"),
        ("i", "p", "l", "e"),
        (" ", "o", "f", " "),
        ("4", ".", ".", "."),
    ]


def test_chunk_with_default_fill_value():
    assert list(chunk(4, range(10))) == [(0, 1, 2, 3), (4, 5, 6, 7), (8, 9, None, None)]


def test_chunk_with_fill_value():
    SENTINEL = object()
    assert list(chunk(4, range(6), SENTINEL)) == [
        (0, 1, 2, 3),
        (4, 5, SENTINEL, SENTINEL),
    ]



from time import sleep

import pytest
from pytest import approx

from pysj import Timer


def test_lap_timing():

    timer = Timer()

    timer.start()
    for i in range(1, 4):
        print(i)
        sleep(0.1)
        assert timer.lap() == approx(i / 10, abs=1e-2)

    assert timer.total() == approx(0.3, abs=1e-2)


def test_reset_timing():

    timer = Timer()

    timer.start()
    for _ in range(1, 4):
        sleep(0.1)
        assert timer.reset() == approx(0.1, abs=1e-2)

    assert timer.total() == approx(0.3, abs=1e-2)


def test_use_as_contextmanager(capsys):

    with Timer():
        sleep(0.1)

    out, err = capsys.readouterr()
    output_line_1, output_line_2 = out.splitlines()
    assert output_line_1 == "Starting timer"
    assert output_line_2.startswith("Elapsed time")
    assert float(output_line_2[13:17]) == approx(0.1)



# fmt: off
common_image_extensions = {  # Formats from the file support list of pillow
    "png", "jpg", "jpeg", "jfif", "j2p", "jpx", "j2k", "gif", "bmp", "dds", "dib", "eps", "icns", "im",
    "msp", "pcx", "apng", "pbm", "pgm", "ppm", "pnm", "sgi", "spi", "tga", "tiff", "tif", "webp", "xbm",
}

all_image_extensions = {
    'dwg', 'im', 'pti', 'tga', 'spng', 'emf', 'jxrs', 'dxf', 'tfx', 'cr2', 'jpg', 'dds', 'uvvi', 'webp',
    'jpe', 'avcs', 'cgm', 'ico', 'xpm', 'jxsc', 'b16', 'heic', 'wbmp', 'heif', 'rlc', 'fits', 'pgm',
    'hsj2', 'jph', 'jfif', 'jxss', 'jxr', 'avci', 'xwd', 'jxs', 'xif', 'svgz', 'hej2', 'svg', 'ppm',
    'dib', 'ktx2', 'jxsi', 'orf', 'jpgm', 'uvg', 'mdi', 'ief', 'jpg2', 'heics', 'pat', 'wmf', 's1n', 'uvi',
    'tiff', 'tif', 'ktx', 'sgi', 'jpf', 'mmr', 'apng', 'rgbe', 'fbs', 'pcx', 'uvvg', 'ras', 'fts', 'djvu',
    'xyze', 'xbm', 'sjpg', 'jp2', 'eps', 'cdr', 'j2k', 'sgif', 'azv', 'nef', 's1g', 'hdr', 'vtf', 'jpx',
    'jls', 'jxra', 'exr', 'btf', 'spi', 'psd', 'drle', 'msp', 'sjp', 'cdt', 'fst', 'erf', 'jng', 'bmp',
    'jpeg', 'jhc', 'crw', 'j2p', 'pnm', 'icns', 'fit', 'tap', 'fpx', 'jpm', 'cpt', 't38', 'heifs', 'jphc',
    'djv', 's1j', 'rgb', 'png', 'pgb', 'pbm', 'gif', 'btif', 'spn'}
# fmt: on


def _get_memetypes_from_os():
    """Todo: maybe use this in a build step"""
    import mimetypes

    mimetypes.init()
    for extension, mimetype in mimetypes.types_map.items():
        if mimetype.startswith("image/"):
            all_image_extensions.add(extension[1:].lower())
